/***************************************************************************
 *            functions.c
 *
 *  Mon Sep  4 15:30:58 2006
 *  Copyright  2006  Vincent Amouret
 *  vincent.amouret@gmail.com
 ****************************************************************************/


#include "functions.h"

#include <glib/gi18n.h>
#include "common_macros.h"
#include <splashycnf.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>

#define SPLASHY_CONFIG_BIN "/sbin/splashy_config"

enum
  {
    DESC_COLUMN,
    PREVIEW_COLUMN,
    N_COLUMNS
  };

GList *get_list_of_themes (void);


void
initialise_parser (void)
{
  /*
   * Initialise the parser
   */
  if (!splashy_init_config (SPL_CONFIG_DIR "/" SPL_CONFIG_FILE_NAME))
    {
      ERROR_PRINT ("%s",
                   _("Error occured while starting Splashy\n"
                     "Make sure that you can read Splashy's configuration file\n"));
    }
}


GdkPixbuf *
create_pixbuf (const gchar * filename)
{
  GdkPixbuf *pixbuf;
  GError *error = NULL;

  if (!filename || !filename[0])
    return NULL;

  pixbuf = gdk_pixbuf_new_from_file (filename, &error);
  if (!pixbuf)
    {
      fprintf (stderr, "Failed to load pixbuf file: %s: %s\n", filename, error->message);
      g_error_free (error);
    }

  return pixbuf;
}

GList *
get_list_of_themes (void)
{
  const gchar *_theme = NULL;
  const gchar *themes_dir;
  gchar *_theme_config;
  GList *tmp_list = NULL;

  themes_dir = splashy_get_config_string (SPL_THEMES_DIR);
  GError *ErrorDir = NULL;
  GDir *_dir = g_dir_open (themes_dir, 0, &ErrorDir);
  if (ErrorDir == NULL)
    {
      while ((_theme = g_dir_read_name (_dir)))
        {
          _theme_config = g_build_filename (themes_dir, _theme, SPL_THEME_CONFIG_FILE_NAME, NULL);

          if (g_file_test (_theme_config, G_FILE_TEST_IS_REGULAR))
            {
              tmp_list = g_list_append (tmp_list, g_path_get_basename (_theme));
            }
        }
      g_dir_close (_dir);
    }
  else
    {
      g_assert (_dir == NULL);
      fprintf (stderr, "%s\n", ErrorDir->message);
      g_error_free (ErrorDir);
    }

  return g_list_sort (tmp_list, (GCompareFunc) g_strcasecmp);
}

void
draw_theme_list (GtkWidget * treeview)
{

  ListOfThemes = get_list_of_themes ();

  GtkListStore *list_store = NULL;
  GtkTreeIter iter;
  GdkPixbuf *pix;
  GdkPixbuf *pix_scaled;
  const gchar *current_theme;
  gint current_theme_index = 0;
  gint i = 0;
  gchar *pix_filename;

  list_store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, GDK_TYPE_PIXBUF);

  current_theme = splashy_get_config_string (SPL_CURRENT_THEME);

  for (i = 0; i < g_list_length (ListOfThemes); i++)
    {
      /*
       * Change the theme.xml file
       */
      splashy_change_config_file (g_build_filename (splashy_get_config_string (SPL_THEMES_DIR),
                                                    g_list_nth_data (ListOfThemes, i), SPL_THEME_CONFIG_FILE_NAME,
                                                    NULL));

      pix_filename = g_build_filename (splashy_get_config_string (SPL_THEMES_DIR), g_list_nth_data (ListOfThemes, i),
                                       g_basename (splashy_get_config_string ("/splashy/background/boot")), NULL);
      pix = create_pixbuf (pix_filename);
      pix_scaled = gdk_pixbuf_scale_simple (pix, 102, 76, GDK_INTERP_BILINEAR);

      /*
       * Add a new row to the model 
       */
      gtk_list_store_append (list_store, &iter);
      gtk_list_store_set (list_store, &iter, DESC_COLUMN,
                          g_strconcat ("<b>", g_list_nth_data (ListOfThemes, i), "</b>\n",
                                       g_markup_escape_text (splashy_get_config_string ("/splashy/info/description"), -1),
                                       "\n", "<i>",
                                       g_markup_escape_text (splashy_get_config_string ("/splashy/info/author"), -1),
                                       "</i>", NULL),
                          PREVIEW_COLUMN, pix_scaled, -1);

      if (g_strcasecmp (current_theme, g_list_nth_data (ListOfThemes, i)) == 0)
        current_theme_index = i;

      g_free (pix_filename);
      g_object_unref (pix);
      g_object_unref (pix_scaled);
    }

  gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (list_store));

  /*
   * Reset the theme.xml file 
   */
  splashy_change_config_file (g_build_filename (splashy_get_config_string (SPL_THEMES_DIR),
                               splashy_get_config_string (SPL_CURRENT_THEME), SPL_THEME_CONFIG_FILE_NAME, NULL));

  /*
   * Select the current theme
   */
  gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), gtk_tree_path_new_from_indices (current_theme_index), NULL, FALSE);
  gtk_widget_grab_focus (treeview);

}

void
format_theme_list (GtkWidget * widget)
{
  GtkTreeViewColumn *column;
  GtkCellRenderer *renderer;
  /*
   * Format the tree view
   */
  renderer = gtk_cell_renderer_pixbuf_new ();
  column = gtk_tree_view_column_new_with_attributes ("Preview", renderer, "pixbuf", PREVIEW_COLUMN, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);

  renderer = gtk_cell_renderer_text_new ();

  column = gtk_tree_view_column_new_with_attributes ("Description", renderer, "markup", DESC_COLUMN, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
}

/**
 * @desc Exec splashy_config, this function is blocant
 * @param argv List of the options passed to splashy_config, terminated by NULL
 * @see execv
 */
gint
splashy_config (gchar * argv[])
{
  pid_t pid;
  gint status;
  if (!(pid = fork ()))
    {
      if (execv (SPLASHY_CONFIG_BIN, argv) == -1)
        {
          fprintf (stderr, "splashy_config failed to start\n");
          perror ("execv()");
          _exit (1);
        }
    }
  if (pid == -1)
    fprintf (stderr, "fork() failed, the back end hasn't been launched.");

  /*
   * Waiting the child...
   */
  waitpid (pid, &status, 0);

  return WEXITSTATUS (status);
}
